@@ -11,9 +11,9 @@ different representations for an IPv6 address:
 
 =over 1
 
-=item C<add_vec>/C<add_vec_range> accepts a 128-bit L<Bit::Vector|Bit::Vector> object for an address
+=item * C<add_vec>/C<add_vec_range> accepts a 128-bit L<Bit::Vector|Bit::Vector> object for an address
 
-=item C<add_str>/C<add_str_range> takes a packed string as returned by C<Socket::unpack_sockaddr_in6>
+=item * C<add_str>/C<add_str_range> takes a packed string as returned by C<Socket::unpack_sockaddr_in6>
 
 =back
 
@@ -42,12 +42,11 @@ package Net::CIDR::Lookup::IPv6;
 use strict;
 use warnings;
 use Carp;
-$Carp::Verbose=1;
-use Socket qw/ inet_ntop inet_pton AF_INET6 /;
+use Socket qw/ getaddrinfo unpack_sockaddr_in6 inet_ntop AF_INET6 /;
 use Bit::Vector;
 use parent 'Net::CIDR::Lookup';
 
-our $VERSION = '0.51';
+our $VERSION = '0.5';
 
 =head2 add
 
@@ -199,7 +198,7 @@ Like C<lookup()> but accepts the address as a Bit::Vector object.
 
 =cut
 
-sub lookup_vec { shift->_lookup($_[0]->Clone) }   ## no critic (Subroutines::RequireArgUnpacking)
+sub lookup_vec { _lookup($_[0], $_[1]->Clone) }   ## no critic (Subroutines::RequireArgUnpacking)
 
 =head2 lookup_str
 
@@ -212,7 +211,7 @@ C<Socket::unpack_sockaddr_in6>.
 
 =cut
 
-sub lookup_str { shift->_lookup(_str2vec($_[0])) }   ## no critic (Subroutines::RequireArgUnpacking)
+sub lookup_str { _lookup(_str2vec($_[0])) }   ## no critic (Subroutines::RequireArgUnpacking)
 
 =head2 to_hash
 
@@ -271,17 +270,17 @@ sub walk { $_[0]->_walk(Bit::Vector->new(128), 0, $_[1]) }   ## no critic (Subro
 
 =over 1
 
-=item The IPv6 version hasn't seen any real-world testing and the unit tests
+=item * The IPv6 version hasn't seen any real-world testing and the unit tests
 are still rather scarce, so there will probably be more bugs than listed here.
 
-=item I didn't need deletions yet and deleting parts of a CIDR block is a bit more
+=item * I didn't need deletions yet and deleting parts of a CIDR block is a bit more
 complicated than anything this class does so far, so it's not implemented.
 
-=item Storing an C<undef> value does not work and yields an error. This would be
+=item * Storing an C<undef> value does not work and yields an error. This would be
 relatively easy to fix at the cost of some memory so that's more a design
 decision.
 
-=item A consequence of the same design is also that a /0 block can't be formed.
+=item * A consequence of the same design is also that a /0 block can't be formed.
 Although it doesn't make much sense, this might happen if your input is that
 weird.
 
@@ -383,8 +382,11 @@ sub _str2vec {   ## no critic (Subroutines::RequireArgUnpacking)
 
 # Parse an IPv6 address and return a Bit::Vector object
 sub _parse_address {   ## no critic (Subroutines::RequireArgUnpacking)
+    my ($err, @result) = getaddrinfo($_[0], 0);
+    $err and croak "Error parsing address ($_[0]): $err";
+    # Some of this could be replaced by _str2vec but isn't for speed
     my $b = Bit::Vector->new(128);
-    $b->Chunk_List_Store(32, reverse unpack 'N4', inet_pton(AF_INET6, shift));
+    $b->Chunk_List_Store(32, reverse unpack 'N4', (unpack_sockaddr_in6 $result[0]{addr})[1]);
     return $b;
 }
 
@@ -394,34 +396,27 @@ sub _addr2print { inet_ntop(AF_INET6, pack('N4', reverse $_[0]->Chunk_List_Read(
 # Walk the tree in depth-first LTR order
 sub _walk {
 	my ($node, $addr, $bits, $cb) = @_;
-	my ($l, $r, $rightflag);
-    my @node_stack = ($node, 0, $bits);
+	my ($l, $r);
+    my @node_stack = ($bits, $node);
     #print "================== WALK ==================: ", join(':',caller),"\n"; 
-    while(@node_stack) {
-        ($node, $rightflag, $bits) = splice @node_stack, -3; # pop 3 elems 
+    while(defined($node = pop @node_stack)) {
+        $bits    = pop @node_stack;
         #print "LOOP: stack size ".@node_stack."\n";
-
-        $addr->Bit_On(128-$bits) if $rightflag;
-
         if(__PACKAGE__ eq ref $node) {
             ($l, $r) = @$node;
-            #printf "Popped [%s, %s]:%s/%d\n",
-            #    ($l//'') =~ /^Net::CIDR::Lookup::IPv6=/ ? '<node>' : $l//'<undef>',
-            #    ($r//'') =~ /^Net::CIDR::Lookup::IPv6=/ ? '<node>' : $r//'<undef>',
-            #    _addr2print($addr), $bits;
+            #printf "Popped l=%s r=%s, bits=%d\n", ($l//'<undef>'), ($r//'<undef>'), $bits;
             ++$bits;
 
             # Check left side
             $addr->Bit_Off(128 - $bits);
             if(__PACKAGE__ eq ref $l) {
-                #defined $r and print "L: pushing right node=$r, bits=$bits\n";
-                defined $r and push @node_stack, ($r, 1, $bits);
-                #defined $r and print "L: pushing left  node=$l, bits=$bits\n";
-                push @node_stack, ($l, 0, $bits);
-                #printf "L: addr=%032b (%s)\n", $addr, _addr2print($addr);
+                #print "L: pushing node=$l, bits=$bits\n";
+                defined $r and push @node_stack, ($bits, $r);
+                push @node_stack, ($bits, $l);
+                #printf "L: addr=%032b (%s)\n", $addr, _int2dq($addr);
                 next; # Short-circuit back to loop w/o checking $r!
             } else {
-                #defined $l and printf "L: CALLBACK (%s/%d) => %s\n", _addr2print($addr), $bits, $l;
+                #defined $l and printf "L: CALLBACK (%s/%d) => %s\n", _int2dq($addr), $bits, $l;
                 defined $l and $cb->($addr, $bits, $l);
             }
         } else {
@@ -434,11 +429,11 @@ sub _walk {
         # Check right side
         $addr->Bit_On(128 - $bits);
         if(__PACKAGE__ eq ref $r) {
-            #print "R: pushing right node=$r, bits=$bits\n";
-            push @node_stack, ($r, 1, $bits);
+            #print "R: pushing node=$r, bits=$bits\n";
+            push @node_stack, ($bits, $r);
             #printf "R: addr=%032b (%s)\n", $addr, _addr2print($addr);
         } else {
-            #defined $r and printf "R: CALLBACK (%s/%d) => %s\n", _addr2print($addr), $bits, $r;
+            #defined $r and printf "R: CALLBACK (%s/%d) => %s\n", _addr2prin($addr | 1 << 32-$bits), $bits, $r;
             defined $r and $cb->($addr, $bits, $r);
         }
     }
